@@ -65,9 +65,9 @@ static void build_hash_table(struct sum_struct *s)
   int i;
 
   if (!tag_table)
-    tag_table = (int *)malloc(sizeof(tag_table[0])*TABLESIZE);
+    tag_table = new_array(int, TABLESIZE);
 
-  targets = (struct target *)malloc(sizeof(targets[0])*s->count);
+  targets = new_array(struct target, s->count);
   if (!tag_table || !targets) 
     out_of_memory("build_hash_table");
 
